import os
import json

from userclass import User
from repo import Repo
from notification import Notification
from settings import USER_CREDENTIALS, USER_DATA, USERNAME, NOTIFIER

def dump_user_data(user):
    """ Function to store the User object in a JSON file.

    Parameters
    ==========
    user: User
        The object to be stored. 
    
    """
    # create the USER_DATA file if it does not already exist.
    with open(USER_DATA, 'w') as f:
        json.dump(user.toJSON(), f)

def load_user_data(user):
    with open(USER_DATA, 'r') as f:
        data = json.loads(json.load(f))
        if data['username'] == user.username:
            userdata = data
        else:
            print('Wrong input') 

    return userdata

def main_handler():
    newuser = User.from_handle(USERNAME) # New User Data
    
    if os.path.isfile(USER_DATA):
        userdata = load_user_data(newuser) # Old user data
    
    else:
        userdata = json.loads(User(USERNAME).toJSON())

    # get followers notification
    old_followers = userdata['followers']
    notifications = newuser.get_notifications(old_followers)
    
    # get repo notifications
    newrepos = newuser.repos
    oldrepos = userdata['repos']
    for newrepo in newrepos:
        if oldrepos is None:
            oldstars, oldwatchers, oldforks = 0, 0, 0
        else:
            for oldrepo in oldrepos:
                if newrepo.name == oldrepo['name']:
                    oldstars, oldwatchers, oldforks = oldrepo['stars'], oldrepo['watchers'], oldrepo['forks']
                    break
            else:
                oldstars, oldwatchers, oldforks = 0, 0, 0

        repo_notifications = newrepo.get_notifications(oldstars, oldwatchers, oldforks)
        # print("Repo: " + str(newrepo))

        notifications += repo_notifications
    
    for notification in notifications:
        #print(notification.message)
        notification.notify()

    dump_user_data(newuser)



import requests

from settings import URL, HEADERS, USERNAME, PASSWORD
from notification import Notification

class Repo(object):

    def __init__(self, name, owner, forks, watchers, stars):
        """ Represents a repository object returned by GitHub API v3.

        Parameters
        ==========
        name: str
            Name of the repository

        owner: str
            Owner of the repository

        forks: int
            Forks count of the repository

        watchers: int
            Watchers count of the repository

        stars: int
            Stars count of the repository

        url: str
            API URL of the repository

        """
        self.name = name
        self.owner = owner
        self.forks = forks
        self.watchers = watchers
        self.stars = stars
        self.url = 'https://github.com/' + owner + '/' + name

    def __repr__(self):
        return '{}({}, {}, {}, {}, {})'.format(self.__class__.__name__,
                                               self.name, self.owner, self.forks,
                                               self.watchers, self.stars)

    @classmethod
    def from_name(cls, repo_name, owner_name):
        url = URL + '/repos/' + owner_name + "/" + repo_name
        # print("Repo = " + repo_name + ' : ' + url)
        repo = requests.get(url, headers=HEADERS, auth=(USERNAME, PASSWORD)).json()
        # handle pagination

        #owner = owner_name
        #name = repo_name
        #forks = repo['forks_count']
        #watchers = repo['watchers_count']
        #stars = repo['stargazers_count']
        #return cls(name, owner, forks, watchers, stars)
        return cls.from_Repository(repo)

    @classmethod
    def from_Repository(cls, repo):
        name = repo['name']
        owner = repo['owner']['login']
        forks = repo['forks_count']
        watchers = repo['watchers_count']
        stars = repo['stargazers_count']
        return cls(name, owner, forks, watchers, stars)

    def get_notifications(self, old_stars, old_watchers, old_forks):
        new_stars, new_watchers, new_forks = self.stars, self.watchers, self.forks

        delta_stars = new_stars - old_stars
        delta_watchers = new_watchers - old_watchers
        delta_forks = new_forks - old_forks

        notifications = []
        if delta_stars != 0:
            context = 'starr'
            if delta_stars < 0:
                context = 'un' + context # not a typo

            notifications.append(Notification.generate_message('{} users'.format(delta_stars), context, self.name))

        if delta_watchers != 0:
            context = 'watch'
            if delta_watchers < 0:
                context = 'un' + context

            notifications.append(Notification.generate_message('{} users'.format(delta_watchers), context, self.name))

        if delta_forks != 0:
            context = 'fork'
            if delta_forks < 0:
                context = 'un' + context

            notifications.append(Notification.generate_message('{} users'.format(delta_watchers), context, self.name))

        return notifications



import subprocess

from settings import NOTIFIER

class Notification(object):
    """ Base class for all notifications.
    Every notification is of the form: "<protagonist> <context> <target>"

    Example
    =======
    In []: from ghnotifier.notification import Notification
    In []: a = Notification("kshitij10496 followed you")
    In []: print(a)
    kshitij10496 followed you
    In []: print(a.target)
    you
    In []: print(a.context)
    follow
    In []: print(a.protagonist)
    kshitij10496

    """
    def __init__(self, message):
        """
        Parameters
        ==========
        message: str
            The notification message

        """
        self.title = 'GitHub Notification'
        self.message = message
        self.target = message.split()[-1]
        self.protagonist = message.split()[:-2]
        self.context = message.split()[-2][:-2]

    @classmethod
    def generate_message(cls, protagonist, context, target):
        """
        Parameters
        ==========
        target: str
            The target user for the notification

        protagonist: str
            The protagonist of the notification

        context: str
            The context of the notification
        
        """
        message = "{} {}ed {}".format(protagonist, context, target)
        return cls(message)

    def __repr__(self):
        return "{}({})".format(self.__class__.__name__, self.message)

    def __str__(self):
        return self.message


    def notify(self):
        """ Notifies the logged in user with an update message using the notifer application based on the Operating System.

        For MacOS : terminal-notifier
            Linux : notify-send

        """
        if NOTIFIER == "terminal-notifier":
            subprocess.run([NOTIFIER, "-title", self.title, "-message",
                                self.message, "-timeout", "10"])

        elif NOTIFIER == 'notify-send':
            subprocess.run([NOTIFIER, self.title, self.message])

        else:
            return -1

        return 1
    ## TODO: Log the notifications



from setuptools import setup, find_packages
# To use a consistent encoding

setup(
    name = 'gh-notifier',

    version = '0.1.0',

    description = "A Desktop notifier for all your social GitHub notifications",

    url = "https://github.com/kshitij10496/gh-notifier",
    author='Kshitij Saraogi',
    author_email='KshitijSaraogi@gmail.com',

    license='MIT',

    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
    ],

    keywords='github',

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),

    install_requires=['requests'],

    entry_points={
        'console_scripts': [
            'gh-notifier=ghnotifier.main_cli:main',
        ],
    },
)



import os
import sys
import subprocess
import json
import getpass

USER_CREDENTIALS = os.path.expanduser('~') + '/.ghnotifier_config.json'
USER_DATA = os.path.expanduser('~') + '/.ghnotifier_data.json'

if os.path.isfile(USER_CREDENTIALS):
    with open(USER_CREDENTIALS) as f:
        CREDENTIALS = json.load(f)

    USERNAME = CREDENTIALS['USERNAME']
    PASSWORD = CREDENTIALS['PASSWORD']

else:
    USERNAME = input("Enter your GitHub username: ")
    PASSWORD = getpass.getpass(prompt="Enter your password: ")
    CREDENTIALS = {
                    "USERNAME": USERNAME,
                    "PASSWORD": PASSWORD 
                    }
            
    with open(USER_CREDENTIALS, 'w') as f:
        json.dump(CREDENTIALS, f)
    

PLATFORM = sys.platform
NOTIFIER = None
if PLATFORM == 'darwin':
    # test if terminal-notifier is present on the system or not
    status = subprocess.run(['which', 'terminal-notifier']).returncode
    if status == 0:
        NOTIFIER = "terminal-notifier"
    else:
        print("Kindly install terminal-notifier for MacOS.") # Raise exception
elif PLATFORM == 'linux':
    NOTIFIER = "notify-send"

else:
    print("Your system is not supported yet.") # Raise exception
        
URL = 'https://api.github.com' 
HEADERS = {'Accept': 'application/vnd.github.v3+json'}



import pytest

from ghscrap.scrapper import user_search_filter, search_location


def test_user_search_filter():
    filterkeys = ['most followed', 'least followed', 'most repos',
                  'fewest repos', 'recently joined', 'oldest to join GitHub']

    filters = [
               {'s': 'followers', 'o': 'desc'}, {'s': 'followers', 'o': 'asc'},
               {'s': 'repositories', 'o': 'desc'}, {'s': 'repositories', 'o': 'asc'},
               {'s': 'joined', 'o': 'desc'}, {'s': 'joined', 'o': 'asc'}
              ]

    for i in range(len(filters)):
        assert user_search_filter(filterkeys[i]) == filters[i]

    with pytest.raises(ValueError):
        user_search_filter("oldest here")


def test_search_location():
    location = 'Kharagpur'
    handles = ['hargup', 'OrkoHunter', 'vivekiitkgp', 'amrav', 'kshitij10496']

    assert search_location(location, max_users=5) == handles
    assert search_location(location, "most followed", max_users=5) == handles



import json
import requests

from notification import Notification
from repo import Repo
from settings import USER_DATA, USERNAME, PASSWORD, URL, HEADERS

class User(object):
    """ Represents a user encapsulating the GitHub User object. """

    def __init__(self, username, followers=0, repos=None):
        """ Initialises a User object.

          Parameters
        ==========
        username: str
            GitHub username/handle of the user

        followers: int
            No. of GitHub followers of the user

        repos: list
            List of all the user's repositories on GitHub.
            This includes repositories owned by the authenticated user, repositories
            where the user is a collaborator, and repositories that the user has
            access to through an organization membership.

        """
        self.username = username
        self.followers = followers
        self.repos = repos

    def __repr__(self):
        return "{}({}, {}, {})".format(self.__class__.__name__, self.username,
                                       self.followers, self.repos)

    @classmethod
    def from_handle(cls, handle):
        """ Alternate constructor for instantiating an User by its handle.

        Parameters
        ==========
        handle: str
            User's GitHub username/handle.

        """
        username = handle
        user = get_blob(handle)
        followers = user['followers']
        repos = get_repos(handle)
        return cls(username, followers, repos)

    def get_notifications(self, old_count):
        # changes in followers
        # notification
        new_count = self.followers
        notifications = []

        if old_count != new_count:
            if self.username == USERNAME:
                notification_url = URL + '/user/followers'
                target = 'you'

            else:
                notification_url = URL + '/users/' + self.username + '/followers'
                target = self.username

            if new_count > old_count:
                params = {'per_page': self.followers}
                start_page = old_count // 100
                end_page = new_count // 100
                followers = []
                with requests.Session() as s:
                    for i in range(start_page + 1, end_page + 2):
                        params['page'] = i
                        new_page = s.get(notification_url, headers=HEADERS,
                                         params=params, auth=(USERNAME, PASSWORD)).json()
                        #print("Length of page {} : {}".format(i, len(new_page)))
                        followers += new_page

                start_index = old_count % 100
                context = "follow"
                #print("Len of followers:" + str(len(followers)))
                #print("Len of list:" + str(len(followers[start_index:])))
                for user_blob in followers[start_index:]:
                    protagonist = user_blob['login']
                    notifications += [Notification.generate_message(protagonist, context, target)]

            else:
                message = '{} users unfollowed {}'.format(old_count - new_count, target)
                notifications += [Notification(message)]

        return notifications

    def toJSON(self):
        return json.dumps(self, default=lambda o: o.__dict__, sort_keys=True, indent=4)

def get_blob(handle):
    if handle == USERNAME:
        user_url = URL + '/user'

    else:
        user_url = URL + '/users/' + handle

    # print("User = " + USERNAME + " : " + user_url)
    user = requests.get(user_url, headers=HEADERS, auth=(USERNAME, PASSWORD)).json()
    return user

def get_repos(handle):
    if handle == USERNAME:
        repo_url = URL + '/user/repos'

    else:
        repo_url = URL + '/users/' + handle + '/repos'

    repo_response = requests.get(repo_url, headers=HEADERS, auth=(USERNAME, PASSWORD)).json()
    repos = [Repo.from_Repository(repo) for repo in repo_response]
    return repos
